/**
 * WordPress dependencies
 */
const { IconButton, ExternalLink } = wp.components;
const { URLPopover } = wp.editor;
const { Component } = wp.element;
const { __ } = wp.i18n;
const { safeDecodeURI, filterURLForDisplay } = wp.url;

/**
 * Internal dependencies
 */
import PositionedAtSelection from './positioned-at-selection';

const stopKeyPropagation = (event) => event.stopPropagation();

const LinkViewerUrl = ({ url }) => {
	return (
		<ExternalLink
			className={
				'editor-format-toolbar__link-container-value block-editor-format-toolbar__link-container-value'
			}
			href={url}
		>
			{filterURLForDisplay(safeDecodeURI(url))}
		</ExternalLink>
	);
};

const LinkViewer = ({ url, onRemoveLink }) => {
	return (
		// Disable reason: KeyPress must be suppressed so the block doesn't hide the toolbar
		/* eslint-disable jsx-a11y/no-static-element-interactions */
		<div
			className="editor-format-toolbar__link-container-content block-editor-format-toolbar__link-container-content"
			onKeyPress={stopKeyPropagation}
		>
			<LinkViewerUrl url={url} />
			<IconButton
				icon="editor-unlink"
				label={__('Remove Tasty Link', 'tasty-links')}
				onClick={onRemoveLink}
			/>
		</div>
		/* eslint-enable jsx-a11y/no-static-element-interactions */
	);
};

class InlineLinkUI extends Component {
	render() {
		const {
			isActive,
			onRemoveLink,
			activeAttributes: { url, remove, style },
			value,
		} = this.props;
		if (!isActive || remove) {
			return null;
		}
		const key = value.start + style;

		return (
			<PositionedAtSelection key={key}>
				<URLPopover>
					<LinkViewer url={url} onRemoveLink={onRemoveLink} />
				</URLPopover>
			</PositionedAtSelection>
		);
	}
}

export default InlineLinkUI;
