<?php
/**
 * Cache adapter.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon;

use Tasty\Framework\Traits\Singleton;

/**
 * Cache adapter.
 */
class Cache {
	use Singleton;

	/**
	 * Cache key.
	 *
	 * @var string
	 */
	private $cache_key = 'tasty_links_amazon_expired_link_ids';

	/**
	 * Time to live for the cache in hours.
	 *
	 * @var int
	 */
	private $ttl = 1;

	/**
	 * Save the IDs to the cache.
	 *
	 * @since 1.3
	 *
	 * @param array $ids Expired IDs to add.
	 *
	 * @return bool
	 */
	public function set_ids( $ids ) {
		return set_site_transient( $this->cache_key, $ids, $this->ttl * HOUR_IN_SECONDS );
	}

	/**
	 * Get the expired IDs from the cache.
	 *
	 * @since 1.3
	 *
	 * @return array
	 */
	public function get_ids() {
		$ids = get_site_transient( $this->cache_key );
		if ( false === $ids ) {
			return array();
		}
		return $ids;
	}

	/**
	 * Remove the IDs from the cache.
	 *
	 * @since 1.3
	 *
	 * @param array $ids IDs to remove.
	 *
	 * @return void
	 */
	public function remove_ids( $ids ) {
		$current_ids = $this->get_ids();
		if ( empty( $current_ids ) ) {
			return;
		}
		$final_ids = array_diff( $current_ids, $ids );
		$this->set_ids( $final_ids );
	}
}
