<?php
/**
 * Amazon Get Items Resource list.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Amazon;

use Tasty\Framework\Traits\Singleton;
use Tasty_Links\Admin;

/**
 * Handle the Amazon configuration.
 */
class Configuration {
	use Singleton;

	/**
	 * The Amazon options.
	 *
	 * @var array
	 */
	private $storage = array(
		'api_key'     => Admin::AMAZON_API_KEY_OPTION,
		'api_secret'  => Admin::AMAZON_API_SECRET_OPTION,
		'tracking_id' => Admin::AMAZON_TRACKING_ID_OPTION,
	);

	/**
	 * Get the API key.
	 *
	 * @param string $key The setting to get.
	 *
	 * @return string
	 */
	public function __get( $key ) {
		if ( ! array_key_exists( $key, $this->storage ) ) {
			return null;
		}

		$option_value = get_option( $this->storage[ $key ], null );
		if ( empty( $option_value ) ) {
			return null;
		}
		return $option_value;
	}

	/**
	 * Check if the API keys are set.
	 *
	 * @return bool
	 */
	public function has_api_keys() {
		$api_key    = $this->api_key;
		$api_secret = $this->api_secret;
		return ! empty( $api_key ) && ! empty( $api_secret );
	}
}
