<?php
/**
 * Enhancements to the WordPress backend.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links;

use Tasty\Framework\Utils\Url;
use Tasty\Framework\Utils\Vars;

/**
 * Enhancements to the WordPress backend.
 */
class Admin {

	/**
	 * Capability required to manage settings.
	 *
	 * @var string
	 */
	const CAPABILITY = 'edit_others_posts';

	/**
	 * Post meta key used to store link.
	 *
	 * @var string
	 */
	const LINK_KEY = 'ta_link';

	/**
	 * Meta key for amazon ASIN.
	 *
	 * @var string
	 */
	const ASIN_KEY = 'ta_asin';

	/**
	 * Key used for the meta box nonce.
	 *
	 * @var string
	 */
	const NONCE = 'tasty-nonce';

	/**
	 * Name of the plugin author.
	 *
	 * @var string
	 */
	const ITEM_AUTHOR = 'Tasty Links';

	/**
	 * Name of the product.
	 *
	 * @var string
	 */
	const ITEM_NAME = 'Tasty Links';

	/**
	 * Group used for authentication settings.
	 *
	 * @var string
	 */
	const SETTINGS_GROUP_LICENSE = 'tasty-links-settings-license';

	/**
	 * Key used for the license check cache.
	 *
	 * @var string
	 */
	const LICENSE_CHECK_CACHE_KEY = 'tasty-links-license-check';

	/**
	 * Option used to store the license key.
	 *
	 * @var string
	 */
	const LICENSE_KEY_OPTION = 'tasty_links_license_key';

	/**
	 * Group used for configuration settings.
	 *
	 * @var string
	 */
	const SETTINGS_GROUP_CONFIG = 'tasty-links-settings-config';

	/**
	 * Section used for configuration settings.
	 *
	 * @var string
	 */
	const SETTINGS_SECTION_CONFIG = 'tasty-links-config';

	/**
	 * Section used for Amazon settings.
	 *
	 * @var string
	 */
	const SETTINGS_SECTION_AMAZON = 'tasty-links-amazon';

	/**
	 * Option used to store the disclosure text.
	 *
	 * @var string
	 */
	const DISCLOSURE_TEXT_OPTION = 'tasty_links_disclosure_text';

	/**
	 * Option used to store the disclosure location.
	 *
	 * @var string
	 */
	const DISCLOSURE_LOCATION_OPTION = 'tasty_links_disclosure_location';

	/**
	 * Option used to set the limit on links for each keyword.
	 *
	 * @var string
	 */
	const MAX_LINK_OPTION = 'tasty_links_max_link';

	/**
	 * Option used to store the disclosure location.
	 *
	 * @var string
	 */
	const SHAREASALE_OPTION = 'tasty_links_shareasale';

	/**
	 * Option used to store Amazon API Key.
	 *
	 * @var string
	 */
	const AMAZON_API_KEY_OPTION = 'tasty_links_amazon_api_key';

	/**
	 * Option used to store Amazon API secret.
	 *
	 * @var string
	 */
	const AMAZON_API_SECRET_OPTION = 'tasty_links_amazon_api_secret';

	/**
	 * Option used to store Amazon tracking ID.
	 *
	 * @var string
	 */
	const AMAZON_TRACKING_ID_OPTION = 'tasty_links_amazon_tracking_id';

	/**
	 * URL for the EDD store.
	 *
	 * @var
	 */
	const STORE_URL = 'https://www.wptasty.com';

	/**
	 * Parent page for the settings page.
	 *
	 * @var string
	 */
	const PAGE_BASE = 'edit.php?post_type=tasty_link';

	/**
	 * Slug for the settings page.
	 *
	 * @var string
	 */
	const PAGE_SLUG = 'tasty-links';

	/**
	 * Meta key for the rel="nofollow" option.
	 *
	 * @var string
	 */
	const REL_NOFOLLOW_META_KEY = 'tasty_links_rel_nofollow';

	/**
	 * Meta key for the rel="sponsored" option.
	 *
	 * @var string
	 */
	const REL_SPONSORED_META_KEY = 'tasty_links_rel_sponsored';

	/**
	 * Meta key for the "Prevent Auto-Linking" option.
	 *
	 * @var string
	 */
	const PREVENT_AUTO_LINKING_META_KEY = 'tasty_links_prevent_auto_linking';

	/**
	 * Meta key for the open in new window option.
	 *
	 * @var string
	 */
	const OPEN_NEW_META_KEY = 'tasty_links_open_new';

	/**
	 * Meta key for the append disclosure option.
	 *
	 * @var string
	 */
	const APPEND_DISCLOSURE_META_KEY = 'tasty_links_append_disclosure';

	/**
	 * Meta key for the optional image URL field.
	 *
	 * @var string
	 */
	const IMAGE_URL_META_KEY = 'tasty_links_image_url';

	/**
	 * Meta key for the optional image URL height field.
	 *
	 * @var string
	 */
	const IMAGE_URL_HEIGHT_META_KEY = 'tasty_links_image_url_height';

	/**
	 * Meta key for the optional image URL width field.
	 *
	 * @var string
	 */
	const IMAGE_URL_WIDTH_META_KEY = 'tasty_links_image_url_width';

	/**
	 * Meta key for Amazon cache expiry.
	 *
	 * @var string
	 */
	const AMAZON_EXPIRY_META_KEY = 'tasty_links_amazon_expiry';

	/**
	 * Meta key for Amazon invalid products.
	 *
	 * @var string
	 */
	const AMAZON_INVALID_META_KEY = 'tasty_links_amazon_invalid';

	/**
	 * Option name for plugin activation state.
	 *
	 * @var string
	 */
	const PLUGIN_ACTIVATION_OPTION = 'tasty_links_do_activation_redirect';

	/**
	 * Option used to set where to replace links.
	 *
	 * @var string
	 */
	const LINK_LOCATION_OPTION = 'tasty_links_post_types';

	/**
	 * Initialize the upgrader
	 */
	public static function action_admin_init() {
		new Updater(
			self::STORE_URL,
			TASTY_LINKS_PLUGIN_FILE,
			array(
				'version'   => TASTY_LINKS_PLUGIN_VERSION,
				'license'   => get_option( self::LICENSE_KEY_OPTION ),
				'item_name' => self::ITEM_NAME,
				'author'    => self::ITEM_AUTHOR,
			)
		);
		add_editor_style( plugins_url( 'assets/css/editor.css', __DIR__ ) );

		if ( get_option( self::PLUGIN_ACTIVATION_OPTION, false ) ) {
			delete_option( self::PLUGIN_ACTIVATION_OPTION );

			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if ( ! isset( $_GET['activate-multi'] ) ) {
				$url = menu_page_url( self::PAGE_SLUG, false );
				$url = add_query_arg( array( 'tab' => 'about' ), $url );
				wp_safe_redirect( str_replace( '#038;', '&', $url ) );
				exit;
			}
		}

		add_action( 'tasty_admin_notices', array( __CLASS__, 'maybe_show_redirect_notice' ) );
	}

	/**
	 * Actions to perform when activating the plugin.
	 */
	public static function plugin_activation() {
		update_option( self::PLUGIN_ACTIVATION_OPTION, true );
	}

	/**
	 * Includes PHP and plugin versions in the user agent for update checks.
	 *
	 * @param array  $r   An array of HTTP request arguments.
	 * @param string $url The request URL.
	 * @return array
	 */
	public static function filter_http_request_args( $r, $url ) {
		if ( self::STORE_URL !== $url
			|| 'POST' !== $r['method']
			|| empty( $r['body'] )
			|| ! is_array( $r['body'] )
			|| empty( $r['body']['item_name'] )
			|| self::ITEM_NAME !== $r['body']['item_name'] ) {
			return $r;
		}

		$r['user-agent'] = rtrim( $r['user-agent'], ';' )
			. '; PHP/' . PHP_VERSION . '; '
			. self::ITEM_NAME . '/' . TASTY_LINKS_PLUGIN_VERSION;
		return $r;
	}

	/**
	 * Modifications to the main query.
	 *
	 * @param WP_Query $query WP_Query object.
	 */
	public static function action_pre_get_posts( $query ) {
		if ( ! is_admin() || ! $query->is_main_query() ) {
			return;
		}
		if ( Content_Model::POST_TYPE !== $query->get( 'post_type' ) ) {
			return;
		}

		$query->set( 'orderby', 'title' );
		$query->set( 'order', 'ASC' );
	}

	/**
	 * Enqueue any scripts and styles.
	 */
	public static function action_admin_enqueue_scripts() {
		$screen = get_current_screen();
		if ( ! empty( $screen->post_type )
			&& Content_Model::POST_TYPE === $screen->post_type ) {
			$assets_path = dirname( __DIR__ ) . '/assets';
			$assets_url  = plugins_url( 'assets', __DIR__ );
			$filepath    = '/js/amazon-conversion-modal.js';
			$mtime       = filemtime( $assets_path . $filepath );
			wp_enqueue_script(
				'tasty-links-amazon-conversion-modal',
				$assets_url . $filepath . '?v=' . (int) $mtime,
				array( 'jquery' ),
				TASTY_LINKS_PLUGIN_VERSION,
				true
			);
			wp_localize_script(
				'tasty-links-amazon-conversion-modal',
				'tastyLinksAmazonConversionModal',
				array(
					'pageTitleActionLabel' => esc_html__( 'Add Amazon Link', 'tasty-links' ),
				)
			);
			$filepath = '/css/amazon-conversion-modal.css';
			$mtime    = filemtime( $assets_path . $filepath );
			wp_enqueue_style(
				'tasty-links-amazon-conversion-modal',
				$assets_url . $filepath . '?v=' . (int) $mtime,
				array(),
				TASTY_LINKS_PLUGIN_VERSION
			);
			if ( ! has_action( 'admin_footer', array( __CLASS__, 'handle_amazon_conversion_modal' ) ) ) {
				add_action( 'admin_footer', array( __CLASS__, 'handle_amazon_conversion_modal' ) );
			}
		}
	}

	/**
	 * Register admin menu UX.
	 */
	public static function action_admin_menu() {
		register_setting(
			self::SETTINGS_GROUP_LICENSE,
			self::LICENSE_KEY_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::DISCLOSURE_TEXT_OPTION,
			array(
				'sanitize_callback' => 'wp_kses_post',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::DISCLOSURE_LOCATION_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::MAX_LINK_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::SHAREASALE_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::AMAZON_API_KEY_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::AMAZON_API_SECRET_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::AMAZON_TRACKING_ID_OPTION,
			array(
				'sanitize_callback' => 'sanitize_text_field',
			)
		);
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			self::LINK_LOCATION_OPTION,
			array(
				'sanitize_callback' => array( 'Tasty\Framework\Utils\Vars', 'sanitize_item' ),
			)
		);
	}

	/**
	 * Filter columns in the Manage Posts view.
	 *
	 * @param array  $columns   Columns and their names.
	 * @param string $post_type Post type.
	 */
	public static function filter_manage_posts_columns( $columns, $post_type ) {
		if ( Content_Model::POST_TYPE !== $post_type ) {
			return $columns;
		}
		$new_columns = array();
		foreach ( $columns as $key => $title ) {
			if ( 'title' === $key ) {
				$title = esc_html__( 'Keyword', 'tasty-links' );
			}
			$new_columns[ $key ] = $title;
			if ( 'title' === $key ) {
				$new_columns[ self::LINK_KEY ] = esc_html__( 'Link', 'tasty-links' );
			}
		}
		return $new_columns;
	}

	/**
	 * Display any custom columns.
	 *
	 * @param string  $name    Column name.
	 * @param integer $post_id Post object.
	 */
	public static function action_manage_posts_custom_column( $name, $post_id ) {
		switch ( $name ) {
			case self::LINK_KEY:
				$value = get_post_meta( $post_id, self::LINK_KEY, true );
				if ( ! $value ) {
					break;
				}
				$display = $value;
				if ( strlen( $value ) > 50 ) {
					$display = substr( $display, 0, 50 ) . '&hellip;';
				}
				echo '<a href="' . esc_url( $value ) . '" target="_blank">' . esc_html( $display ) . '</a>';
				break;
		}
	}

	/**
	 * Display an admin notice when the license key is incorrect.
	 *
	 * @deprecated 1.2
	 */
	public static function action_admin_notices_license_key() {
		_deprecated_function( __METHOD__, '1.2' );
	}

	/**
	 * Activate the license when the option is updated.
	 *
	 * @deprecated 1.2
	 */
	public static function action_update_option_register_license() {
		_deprecated_function( __METHOD__, '1.2' );
	}

	/**
	 * Clear license key cache when the option is updated.
	 */
	public static function action_update_option_clear_transient() {
		delete_transient( self::LICENSE_CHECK_CACHE_KEY );
		$cache_key = md5( 'edd_plugin_' . sanitize_key( plugin_basename( TASTY_LINKS_PLUGIN_FILE ) ) . '_version_info' );
		delete_site_transient( $cache_key );

		// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_serialize
		$cache_key = 'edd_api_request_' . substr( md5( serialize( basename( TASTY_LINKS_PLUGIN_FILE, '.php' ) ) ), 0, 15 );
		delete_site_transient( $cache_key );
		delete_site_transient( 'update_plugins' );
	}

	/**
	 * Include the 'Settings' and 'Remove license' links in plugin actions
	 *
	 * @param array $links Existing plugin actions.
	 * @return array
	 */
	public static function filter_plugin_action_links( $links ) {
		$links['tl_settings']        = '<a href="' . esc_url( menu_page_url( self::PAGE_SLUG, false ) ) . '">' . esc_html__( 'Settings', 'tasty-links' ) . '</a>';
		$links['tl_license_manager'] = '<a href="' . esc_url( Url::get_main_admin_url() ) . '">' . esc_html__( 'License manager', 'tasty-links' ) . '</a>';
		return $links;
	}

	/**
	 * Handle AJAX request to remove the license key.
	 *
	 * @deprecated 1.2
	 */
	public static function handle_wp_ajax_remove_license_key() {
		_deprecated_function( __METHOD__, '1.2' );
	}

	/**
	 * Handle POST request to convert Amazon HTML into a Tasty Link.
	 */
	public static function handle_admin_post_convert_amazon_html() {
		$nonce = Vars::get_param( 'nonce' );
		if ( ! wp_verify_nonce( $nonce, 'tasty_links_convert_amazon_html' ) ) {
			wp_die( esc_html__( "Sorry, you don't have permission to do this.", 'tasty-links' ) );
		}

		// This is sanitized in the Content_Model::get_link_image_from_html() method.
		// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$html = ! empty( $_POST['tasty_links_amazon_html'] ) ? stripslashes_deep( $_POST['tasty_links_amazon_html'] ) : '';

		$data = Content_Model::get_link_image_from_html( $html );
		if ( empty( $data ) || empty( $data['asin'] ) ) {
			wp_die(
				esc_html__( 'We were unable to parse the Amazon link. To help improve the parser, please share your code snippet with WP Tasty Support.', 'tasty-links' ),
				'',
				array(
					'back_link' => true,
				)
			);
		}
		$post_id = wp_insert_post(
			array(
				'post_type'   => Content_Model::POST_TYPE,
				'post_status' => 'draft',
				'post_title'  => isset( $data['title'] ) ? $data['title'] : '',
			)
		);
		if ( ! $post_id ) {
			wp_die(
				esc_html__( 'Failed to create a Tasty Link for an unexpected reason.', 'tasty-links' ),
				'',
				array(
					'back_link' => true,
				)
			);
		}

		$data['post_id'] = $post_id;

		update_post_meta( $post_id, self::LINK_KEY, $data['link'] );
		update_post_meta( $post_id, self::ASIN_KEY, $data['asin'] );
		update_post_meta( $post_id, self::REL_SPONSORED_META_KEY, 'on' );

		$error = Content_Model::maybe_get_error_message( $data );

		$redirect_to = get_edit_post_link( $post_id, 'raw' );
		if ( empty( $error ) ) {
			update_post_meta( $post_id, self::IMAGE_URL_META_KEY, $data['image'] );
			Content_Model::calculate_image_url_height_width( $post_id );
		} else {
			$redirect_to = add_query_arg(
				array(
					'tl-msg' => rawurlencode( $error ),
				),
				$redirect_to
			);
		}

		wp_safe_redirect( $redirect_to );
		exit;
	}

	/**
	 * Render the settings page for disclosure text.
	 */
	public static function handle_settings_page() {
		add_settings_section(
			self::SETTINGS_SECTION_CONFIG,
			__( 'Configuration', 'tasty-links' ),
			false,
			self::PAGE_SLUG
		);
		add_settings_field(
			self::DISCLOSURE_TEXT_OPTION,
			__( 'Disclosure Text', 'tasty-links' ),
			array( __CLASS__, 'render_editor_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CONFIG,
			array(
				'class'       => 'tl-disclosure-text',
				'name'        => self::DISCLOSURE_TEXT_OPTION,
				'description' => __( 'If enabled for the link, this disclosure text will be automatically appended after the link.', 'tasty-links' ),
			)
		);
		add_settings_field(
			self::DISCLOSURE_LOCATION_OPTION,
			__( 'Disclosure Location', 'tasty-links' ),
			array( __CLASS__, 'render_radio_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CONFIG,
			array(
				'name'          => self::DISCLOSURE_LOCATION_OPTION,
				'default_value' => 'link',
				'options'       => array(
					'link'    => __( 'After each link', 'tasty-links' ),
					'content' => __( 'Above the post content', 'tasty-links' ),
				),
			)
		);
		add_settings_field(
			self::MAX_LINK_OPTION,
			__( 'Max Replacements Per Link', 'tasty-links' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CONFIG,
			array(
				'name'        => self::MAX_LINK_OPTION,
				'description' => 'Add a number here if you don\'t want too many links replacements per page.',
			)
		);

		add_settings_field(
			self::LINK_LOCATION_OPTION,
			__( 'Auto-linking Location(s)', 'tasty-links' ),
			array( __CLASS__, 'render_radio_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CONFIG,
			array(
				'name'          => self::LINK_LOCATION_OPTION,
				'class'         => 'wpt-button-container wpt-horizontal',
				'default_value' => array( 'all' ),
				'options'       => self::get_link_locations(),
				'type'          => 'checkbox',
			)
		);

		add_settings_field(
			self::SHAREASALE_OPTION,
			__( 'ShareASale Affiliate ID', 'tasty-links' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_CONFIG,
			array(
				'name'        => self::SHAREASALE_OPTION,
				// translators: Various links.
				'description' => sprintf( __( '<a href="%1$s" target="_blank">Apply for the affiliate program</a>, or <a href="%2$s" target="_blank">find your affiliate ID</a>.', 'tasty-links' ), 'https://www.wptasty.com/affiliates', 'https://www.wptasty.com/affiliate-id' ),
			)
		);
		add_settings_section(
			self::SETTINGS_SECTION_AMAZON,
			__( 'Amazon Associates', 'tasty-links' ),
			array( __CLASS__, 'show_amazon_description' ),
			self::PAGE_SLUG
		);
		add_settings_field(
			self::AMAZON_API_KEY_OPTION,
			__( 'Amazon Access Key', 'tasty-links' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_AMAZON,
			array(
				'name' => self::AMAZON_API_KEY_OPTION,
			)
		);
		add_settings_field(
			self::AMAZON_API_SECRET_OPTION,
			__( 'Amazon Secret Key', 'tasty-links' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_AMAZON,
			array(
				'name' => self::AMAZON_API_SECRET_OPTION,
			)
		);
		add_settings_field(
			self::AMAZON_TRACKING_ID_OPTION,
			__( 'Amazon Store ID', 'tasty-links' ),
			array( __CLASS__, 'render_input_field' ),
			self::PAGE_SLUG,
			self::SETTINGS_SECTION_AMAZON,
			array(
				'name' => self::AMAZON_TRACKING_ID_OPTION,
			)
		);
		self::show_template_part( 'settings' );
	}

	/**
	 * Render the Amazon conversion modal.
	 */
	public static function handle_amazon_conversion_modal() {
		self::show_template_part( 'amazon-conversion-modal' );
	}

	/**
	 * Render the About page.
	 */
	public static function handle_about_page() {
		self::show_template_part( 'about' );
	}


	/**
	 * If the url includes 'error', get the value and add an admin notice.
	 * Used to show error message when new Amazon link fails to fetch API image.
	 *
	 * @since 1.3
	 *
	 * @return void
	 */
	public static function maybe_show_redirect_notice() {
		$error = Vars::get_param( 'tl-msg', 'sanitize_textarea' );
		if ( empty( $error ) || ! Url::is_wpt_page() ) {
			return;
		}

		echo '<div class="tasty-notice tasty-notice-error">' . wp_kses_post( $error ) . '</div>';
	}

	/**
	 * Render a textarea field.
	 *
	 * @param array $args Configuration arguments used by the textarea field.
	 */
	public static function render_editor_field( $args ) {
		$defaults = array(
			'name' => '',
			'rows' => 2,
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) ) {
			return;
		}
		$value = get_option( $args['name'] );
		wp_editor(
			$value,
			$args['name'],
			array(
				'wpautop'       => true,
				'media_buttons' => false,
				'textarea_rows' => $args['rows'],
				'teeny'         => true,
			)
		);
		if ( ! empty( $args['description'] ) ) :
			?>
			<p class="description"><?php echo wp_kses_post( $args['description'] ); ?></p>
			<?php
		endif;
	}

	/**
	 * Render a select field.
	 *
	 * @param array $args Configuration arguments used by the select field.
	 */
	public static function render_radio_field( $args ) {
		$defaults = array(
			'name'          => '',
			'options'       => array(),
			'default_value' => '',
			'type'          => 'radio',
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) || empty( $args['options'] ) ) {
			return;
		}

		$value = get_option( $args['name'], $args['default_value'] );
		self::override_saved_value( $args, $value );

		$multiselect = 'checkbox' === $args['type'];
		if ( $multiselect ) {
			$args['name'] .= '[]';
			if ( empty( $value ) ) {
				$value = $args['default_value'];
			}
		}

		foreach ( $args['options'] as $key => $label ) :
			$id = sanitize_title_with_dashes( $args['name'] . '-' . $key );
			?>
			<label for="<?php echo esc_attr( $id ); ?>" style="display: block;">
				<input type="<?php echo esc_attr( $args['type'] ); ?>" id="<?php echo esc_attr( $id ); ?>" name="<?php echo esc_attr( $args['name'] ); ?>" value="<?php echo esc_attr( $key ); ?>" <?php checked( in_array( $key, (array) $value, true ) ); ?> />
				<?php
				echo '<span>' . esc_html( $label ) . '</span>';
				if ( $key === 'all' && $multiselect ) {
					self::check_all_js( $id );
				}
				?>
			</label>
		<?php endforeach; ?>
		<?php if ( ! empty( $args['description'] ) ) : ?>
			<p class="description"><?php echo wp_kses_post( $args['description'] ); ?></p>
			<?php
		endif;
	}

	/**
	 * If the link location is 'all', then add all the post types to the value so they will be checked.
	 *
	 * @since 1.3
	 *
	 * @param array $args Configuration arguments used by the select field.
	 * @param array $value The saved value of the option.
	 * @return void
	 */
	private static function override_saved_value( $args, &$value ) {
		if ( $args['name'] === self::LINK_LOCATION_OPTION && $value === $args['default_value'] ) {
			$value = array_keys( $args['options'] );
		}
	}

	/**
	 * Check other boxes when one is clicked.
	 *
	 * @since 1.3
	 * @param string $id ID of the checkbox.
	 */
	private static function check_all_js( $id ) {
		?>
		<script>
		document.addEventListener('DOMContentLoaded', function () {
			const wptSelectAllCheckbox = document.getElementById('<?php echo esc_attr( $id ); ?>');
			wptSelectAllCheckbox.addEventListener('change', function () {
				if ( this.checked ) {
					const checkboxes = document.querySelectorAll('input[name="' + this.name + '"]');
					checkboxes.forEach(function (checkbox) {
						checkbox.checked = true;
					});
				}
			});
			/* Uncheck the "All" box if another box is unchecked. */
			const wptCheckboxes = document.querySelectorAll('input[name="' + wptSelectAllCheckbox.name + '"]');
			wptCheckboxes.forEach(function (checkbox) {
				checkbox.addEventListener('change', function () {
					if ( ! this.checked ) {
						wptSelectAllCheckbox.checked = false;
					}
				});
			});
		});
		</script>
		<?php
	}

	/**
	 * Render an input field.
	 *
	 * @param array $args Configuration arguments used by the input field.
	 */
	public static function render_input_field( $args ) {
		$defaults = array(
			'type' => 'text',
			'name' => '',
		);
		$args     = array_merge( $defaults, $args );
		if ( empty( $args['name'] ) ) {
			return;
		}
		$value = get_option( $args['name'] );
		?>
		<input type="<?php echo esc_attr( $args['type'] ); ?>"
			name="<?php echo esc_attr( $args['name'] ); ?>"
			value="<?php echo esc_attr( $value ); ?>"
			class="regular-text"
		/>
		<?php if ( ! empty( $args['description'] ) ) : ?>
			<p class="description"><?php echo wp_kses_post( $args['description'] ); ?></p>
			<?php
		endif;
	}

	/**
	 * Show a description for the Amazon API settings.
	 *
	 * @since 1.3
	 *
	 * @return void
	 */
	public static function show_amazon_description() {
		echo '<p>';
		esc_html_e( 'Enter your Amazon PA API credentials below to automatically get Amazon images and update them daily.', 'tasty-links' );
		printf(
			/* translators: %1$s - opening a link, %2$s - closing a link, %3$s - opening a link, %4$s - closing a link */
			esc_html__( '%1$sFind your API credentials%2$s or %3$slearn more%4$s.', 'tasty-links' ),
			' <a href="https://affiliate-program.amazon.com/assoc_credentials/home" target="_blank" rel="nofollow noopener">',
			'</a>',
			'<a href="https://www.wptasty.com/knowledge-base/how-to-automatically-add-images-from-an-amazon-associates-link" target="_blank" rel="noopener">',
			'</a>'
		);
		echo '</p>';
	}

	/**
	 * Include our TinyMCE button as a TinyMCE plugin, and enqueue a global JS var.
	 *
	 * @param array $plugins Existing registered TinyMCE plugins.
	 * @return array
	 */
	public static function filter_mce_external_plugins( $plugins ) {
		$assets_path           = dirname( __DIR__ ) . '/assets';
		$assets_url            = plugins_url( 'assets', __DIR__ );
		$filepath              = '/js/tinymce-plugin.js';
		$mtime                 = filemtime( $assets_path . $filepath );
		$plugins['tastylinks'] = $assets_url . $filepath . '?v=' . (int) $mtime;
		return $plugins;
	}

	/**
	 * Filter the buttons used in the editor field
	 *
	 * @param array  $buttons    Buttons used in the field.
	 * @param string $editor_id Id for the editor.
	 * @return array
	 */
	public static function filter_teeny_mce_buttons( $buttons, $editor_id ) {
		switch ( $editor_id ) {
			case self::DISCLOSURE_TEXT_OPTION:
				$buttons = array( 'bold', 'italic', 'underline', 'link' );
				break;
		}
		return $buttons;
	}

	/**
	 * Ensure the Tasty Links TinyMCE plugin is properly registered for TeenyMCE instances.
	 *
	 * @param array  $settings Existing TeenyMCE settings.
	 * @param string $editor_id Id for the editor instance.
	 * @return array
	 */
	public static function filter_teeny_mce_before_init( $settings, $editor_id ) {
		if ( 'tasty-recipes-editor' !== $editor_id ) {
			return $settings;
		}
		$external_plugins = array();
		if ( isset( $settings['external_plugins'] ) ) {
			if ( is_string( $settings['external_plugins'] ) ) {
				$decoded_plugins  = json_decode( $settings['external_plugins'], true );
				$external_plugins = $decoded_plugins ? $decoded_plugins : array();
			} elseif ( is_array( $settings['external_plugins'] ) ) {
				$external_plugins = $settings['external_plugins'];
			}
		}
		$assets_path                    = dirname( __DIR__ ) . '/assets';
		$assets_url                     = plugins_url( 'assets', __DIR__ );
		$filepath                       = '/js/tinymce-plugin.js';
		$mtime                          = filemtime( $assets_path . $filepath );
		$external_plugins['tastylinks'] = $assets_url . $filepath . '?v=' . (int) $mtime;
		$settings['external_plugins']   = $external_plugins;
		return $settings;
	}

	/**
	 * Filter the content immediately prior to display
	 *
	 * @param string $content Content to display in the editor.
	 * @return string
	 */
	public static function filter_the_editor_content( $content ) {
		global $post;

		if ( ! tasty_links_should_be_applied( 'editor' ) ) {
			return $content;
		}

		$skipped_links = array();
		if ( $post ) {
			$skipped_links[] = get_permalink( $post->ID );
			$post_type       = get_post_type( $post );

			if ( Frontend::should_disable_auto_linking( $post_type ) ) {
				return $content;
			}
		}
		$content = Frontend::apply_links_to_content( $content, 'editor', null, $skipped_links );
		return $content;
	}

	/**
	 * Render the Tasty Links meta box
	 *
	 * @param WP_Post $post Current post object.
	 */
	public static function action_edit_form_top( $post ) {
		if ( Content_Model::POST_TYPE !== $post->post_type ) {
			return;
		}
		?>
		<h3><?php esc_html_e( 'Keyword', 'tasty-links' ); ?></h3>
		<?php
	}

	/**
	 * Render the Tasty Links meta box
	 *
	 * @param WP_Post $post Current post object.
	 */
	public static function action_edit_form_after_title( $post ) {

		if ( Content_Model::POST_TYPE !== $post->post_type ) {
			return;
		}

		$link = get_post_meta( $post->ID, self::LINK_KEY, true );
		?>
		<p>
			<h3><?php esc_html_e( 'Link', 'tasty-links' ); ?></h3>
			<input type="url" class="widefat" placeholder="<?php esc_attr_e( 'Enter link', 'tasty-links' ); ?>" name="<?php echo esc_attr( self::LINK_KEY ); ?>" value="<?php echo esc_url( $link ); ?>" required>
		</p>
		<?php
		wp_nonce_field( self::NONCE, self::NONCE );
	}

	/**
	 * Register the Tasty Links meta box
	 *
	 * @param string $post_type Post type being rendered.
	 */
	public static function action_add_meta_boxes( $post_type ) {
		if ( Content_Model::POST_TYPE !== $post_type ) {
			return;
		}
		add_meta_box( 'tasty-links', __( 'Options', 'tasty-links' ), array( __CLASS__, 'render_options_meta_box' ), $post_type, 'normal' );
		remove_meta_box( 'slugdiv', $post_type, 'normal' );
	}

	/**
	 * Render the Tasty Links meta box, redux
	 *
	 * @param WP_Post $post Post being edited.
	 */
	public static function render_options_meta_box( $post ) {
		$options = array(
			array(
				'label' => __( 'Add rel="nofollow" to link', 'tasty-links' ),
				'name'  => self::REL_NOFOLLOW_META_KEY,
				'value' => get_post_meta( $post->ID, self::REL_NOFOLLOW_META_KEY, true ),
			),
			array(
				'label' => __( 'Add rel="sponsored" to link', 'tasty-links' ),
				'name'  => self::REL_SPONSORED_META_KEY,
				'value' => get_post_meta( $post->ID, self::REL_SPONSORED_META_KEY, true ),
			),
			array(
				'label' => __( 'Open link in new window', 'tasty-links' ),
				'name'  => self::OPEN_NEW_META_KEY,
				'value' => get_post_meta( $post->ID, self::OPEN_NEW_META_KEY, true ),
			),
			array(
				'label' => __( 'Append disclosure', 'tasty-links' ),
				'name'  => self::APPEND_DISCLOSURE_META_KEY,
				'value' => get_post_meta( $post->ID, self::APPEND_DISCLOSURE_META_KEY, true ),
			),
			array(
				'label' => __( 'Prevent auto-linking', 'tasty-links' ),
				'name'  => self::PREVENT_AUTO_LINKING_META_KEY,
				'value' => get_post_meta( $post->ID, self::PREVENT_AUTO_LINKING_META_KEY, true ),
			),
		);
		foreach ( $options as $option ) :
			if ( empty( $option['value'] ) ) {
				if ( in_array( $option['name'], array( self::REL_SPONSORED_META_KEY, self::PREVENT_AUTO_LINKING_META_KEY ), true ) ) {
					$option['value'] = 'off';
				} else {
					$option['value'] = 'on';
				}
			}
			?>
		<table>
			<tbody>
			<tr>
				<td style="width:190px;font-size:14px;"><?php echo esc_html( $option['label'] ); ?></td>
				<td>
					<label class="tasty-inline-block"><?php esc_html_e( 'On', 'tasty-links' ); ?> <input type="radio" name="<?php echo esc_attr( $option['name'] ); ?>" value="on" <?php checked( 'on', $option['value'] ); ?>></label>
					<label class="tasty-inline-block"><?php esc_html_e( 'Off', 'tasty-links' ); ?> <input type="radio" name="<?php echo esc_attr( $option['name'] ); ?>" value="off" <?php checked( 'off', $option['value'] ); ?>></label>
				</td>
			</tr>
		<?php endforeach; ?>
			</tbody>
		</table>
		<?php
	}

	/**
	 * Add a image URL field to the thumbnail meta box.
	 *
	 * @param string  $html         Existing thumbnail HTML.
	 * @param integer $post_id      ID for the post.
	 * @param integer $thumbnail_id ID for the thumbnail.
	 * @return string
	 */
	public static function filter_admin_post_thumbnail_html( $html, $post_id, $thumbnail_id ) {
		if ( Content_Model::POST_TYPE !== get_post_type( $post_id ) ) {
			return $html;
		}
		$image_url        = get_post_meta( $post_id, self::IMAGE_URL_META_KEY, true );
		$image_url_height = get_post_meta( $post_id, self::IMAGE_URL_HEIGHT_META_KEY, true );
		$image_url_width  = get_post_meta( $post_id, self::IMAGE_URL_WIDTH_META_KEY, true );
		$posted_thumb_id  = Vars::post_param( 'thumbnail_id' );
		$action           = Vars::post_param( 'action' );
		if ( $thumbnail_id
			|| ( 'get-post-thumbnail-html' === $action )
				&& -1 === (int) $posted_thumb_id ) {
			$image_url        = '';
			$image_url_height = '';
			$image_url_width  = '';
		}
		$html .= '<div class="tasty-links-image-url-container">';
		$html .= '<img class="tasty-links-image-preview">';
		$html .= PHP_EOL . PHP_EOL . '<input type="text" value="' . esc_attr( $image_url ) . '" name="' . esc_attr( self::IMAGE_URL_META_KEY ) . '" placeholder="' . esc_attr__( 'Enter image URL', 'tasty-links' ) . '" />';
		$html .= PHP_EOL . PHP_EOL . '<div class="tasty-links-image-hw-container" style="margin-top: 4px;"><label style="width:49%">' . esc_html__( 'Width', 'tasty-links' ) . ' <input style="width:50px" type="text" value="' . esc_attr( $image_url_width ) . '" name="' . esc_attr( self::IMAGE_URL_WIDTH_META_KEY ) . '" /></label>' .
			'<label style="width:49%; margin-left: 4px;">' . esc_html__( 'Height', 'tasty-links' ) . ' <input style="width:50px" type="text" value="' . esc_attr( $image_url_height ) . '" name="' . esc_attr( self::IMAGE_URL_HEIGHT_META_KEY ) . '" /></label></div>';
		$html .= PHP_EOL . PHP_EOL . '<p class="description">' . esc_html__( 'Use an image hosted outside of the WordPress Media Library.', 'tasty-links' ) . '</p>';
		$html .= PHP_EOL . PHP_EOL . '<p class="tasty-links-clear-image-container"><a href="#">' . esc_html__( 'Clear image url', 'tasty-links' ) . '</a></p>';
		$html .= '</div>';
		$html .= <<<EOT
<script>
(function(){
	var thumbnailIdInput = document.querySelector('input[name="_thumbnail_id"]');
	var imageUrlContainer = document.querySelector('.tasty-links-image-url-container');
	var imageHWContainer = document.querySelector('.tasty-links-image-hw-container');
	var imageUrlInput = imageUrlContainer.querySelector('input[type="text"]');
	var imageUrlHeightInput = imageHWContainer.querySelector('input[name="tasty_links_image_url_height"]');
	var imageUrlWidthInput = imageHWContainer.querySelector('input[name="tasty_links_image_url_width"]');
	var imagePreview = imageUrlContainer.querySelector('.tasty-links-image-preview');
	var showHideContainer = function() {
		if ( parseInt( thumbnailIdInput.value ) > 0 ) {
			imageUrlContainer.style.display = 'none';
		} else {
			imageUrlContainer.style.display = 'block';
		}
	};
	var showHideRemoveLink = function() {
		var clearLink = document.querySelector('.tasty-links-clear-image-container');
		var thumbLink = document.querySelector('#set-post-thumbnail');
		if ( imageUrlInput.value.length && parseInt( thumbnailIdInput.value ) <= 0 ) {
			clearLink.style.display = 'block';
			thumbLink.style.display = 'none';
			imagePreview.src = imageUrlInput.value;
			imagePreview.style.display = 'block';
			imageHWContainer.style.display = 'block';
			if (! imageUrlHeightInput.value && ! imageUrlWidthInput.value) {
				var imageObj = new Image();
				imageObj.onload = function() {
					imageUrlHeightInput.value = this.height;
					imageUrlWidthInput.value = this.width;
				}
				imageObj.src = imagePreview.src;
			}
		} else {
			clearLink.style.display = 'none';
			thumbLink.style.display = 'inline';
			imagePreview.src = '';
			imagePreview.style.display = 'none';
			imageHWContainer.style.display = 'none';
			imageUrlHeightInput.value = '';
			imageUrlWidthInput.value = '';
		}
	};
	showHideContainer();
	showHideRemoveLink();
	thumbnailIdInput.addEventListener('change', showHideContainer);
	imageUrlInput.addEventListener('change', showHideRemoveLink);
	document.querySelector('.tasty-links-clear-image-container a').addEventListener('click', function( event ) {
		event.preventDefault();
		imageUrlInput.value = '';
		imageUrlInput.setAttribute('value', '');
		imageUrlInput.dispatchEvent(new Event('change', { bubbles: true }));
	});
}())
</script>
EOT;
		return $html;
	}

	/**
	 * Strip out <span class="tasty-link"> from post content on save.
	 *
	 * Wraps text with <!-- tasty-links-ignore --> when 'data-tasty-link-remove'
	 * is present.
	 *
	 * @param string $content Post content to be saved.
	 * @return string
	 */
	public static function filter_content_save_pre( $content ) {
		if ( false === stripos( $content, 'tasty-link' ) ) {
			return $content;
		}

		$content = preg_replace_callback(
			'#<span[^>]+tasty-link[^>]+>(.+)<\/span>#U',
			function ( $matches ) {
				$text = $matches[1];
				// If 'data-tasty-link-remove' is included in the <span>, then
				// need to transform to the proper HTML comments.
				if ( false !== stripos( $matches[0], 'data-tasty-link-remove' ) ) {
					$text = '<!-- tasty-links-ignore -->' . $text . '<!-- /tasty-links-ignore -->';
				}
				return $text;
			},
			$content
		);
		return $content;
	}

	/**
	 * Save the meta box when the post is saved
	 *
	 * @param integer $post_id ID for the post.
	 */
	public static function action_save_post( $post_id ) {
		$nonce = Vars::post_param( self::NONCE );
		if ( empty( $nonce ) || ! wp_verify_nonce( $nonce, self::NONCE ) ) {
			return;
		}
		$updated_metas = array();
		if ( update_post_meta( $post_id, self::LINK_KEY, Vars::post_param( self::LINK_KEY, 'esc_url_raw' ) ) ) {
			$updated_metas[] = self::LINK_KEY;
		}

		foreach ( array(
			self::REL_NOFOLLOW_META_KEY,
			self::REL_SPONSORED_META_KEY,
			self::OPEN_NEW_META_KEY,
			self::APPEND_DISCLOSURE_META_KEY,
			self::PREVENT_AUTO_LINKING_META_KEY,
		) as $option ) {
			$value = isset( $_POST[ $option ] ) && 'on' === $_POST[ $option ] ? 'on' : 'off';
			update_post_meta( $post_id, $option, $value );
		}

		update_post_meta( $post_id, self::IMAGE_URL_META_KEY, Vars::post_param( self::IMAGE_URL_META_KEY, 'esc_url_raw' ) );
		foreach ( array( self::IMAGE_URL_HEIGHT_META_KEY, self::IMAGE_URL_WIDTH_META_KEY ) as $key ) {
			if ( ! empty( $_POST[ $key ] ) ) {
				$value = absint( $_POST[ $key ] );
			} else {
				$value = '';
			}
			update_post_meta( $post_id, $key, $value );
		}

		do_action( 'tasty_links_after_saving_link', $post_id, $updated_metas );
	}

	/**
	 * Get a rendered template.
	 *
	 * @param string $template Template to render.
	 * @param array  $vars     Variables to pass to the template.
	 * @return string
	 */
	public static function get_template_part( $template, $vars = array() ) {
		$full_path = dirname( __DIR__ ) . '/templates/' . $template . '.php';
		// Provided template may already be a full path.
		if ( ! file_exists( $full_path ) ) {
			$full_path = $template;
		}
		if ( ! file_exists( $full_path ) ) {
			return '';
		}

		ob_start();
		// @codingStandardsIgnoreStart
		if ( ! empty( $vars ) ) {
			extract( $vars );
		}
		// @codingStandardsIgnoreEnd
		include $full_path;
		return ob_get_clean();
	}

	/**
	 * Render a template.
	 *
	 * @since 1.3
	 * @param string $template Template to render.
	 * @param array  $vars     Variables to pass to the template.
	 * @return void
	 */
	public static function show_template_part( $template, $vars = array() ) {
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo self::get_template_part( $template, $vars );
	}

	/**
	 * Add the callback for the plugin menu item.
	 *
	 * @param array $menu_items Admin menu items.
	 *
	 * @return array
	 */
	public static function add_menu_item( $menu_items ) {
		if ( empty( $menu_items['tasty-links'] ) ) {
			return $menu_items;
		}

		$menu_items['tasty-links']['callback'] = array( __CLASS__, 'handle_links_admin_pages' );

		return $menu_items;
	}

	/**
	 * Callback for the newly added menu item.
	 *
	 * @return void
	 */
	public static function handle_links_admin_pages() {
		if ( self::is_current_page( 'about' ) ) {
			self::handle_about_page();
			return;
		}

		self::handle_settings_page();
	}

	/**
	 * Check if current page is one of our admin pages.
	 *
	 * @return bool
	 */
	private static function is_links_admin() {
		$our_page_ids = array(
			'wp-tasty_page_tasty-links',
			'edit-tasty_link',
			'tasty_link',
		);
		return in_array( get_current_screen()->id, $our_page_ids, true );
	}

	/**
	 * Check if current page is the one mentioned.
	 *
	 * @param string $id Page id to check over (about, settings, tasty_link).
	 *
	 * @return bool
	 */
	private static function is_current_page( $id ) {
		switch ( $id ) {
			case 'tasty_link':
				return isset( get_current_screen()->id ) && in_array( get_current_screen()->id, array( 'edit-tasty_link', 'tasty_link' ), true );
			default:
				return 'wp-tasty_page_tasty-links' === get_current_screen()->id
					&& Vars::get_param( 'tab', 'sanitize_text_field', 'settings' ) === $id;
		}
	}

	/**
	 * Register tabs in admin notices area, but after we remove all other admin notices from the framework.
	 *
	 * @return void
	 */
	public static function register_tabs() {
		add_action( 'all_admin_notices', array( __CLASS__, 'add_admin_header' ), 101 );
	}

	/**
	 * Add admin tabs to the head of all links admin pages.
	 *
	 * @return void
	 */
	public static function add_admin_header() {
		if ( ! self::is_links_admin() ) {
			return;
		}

		$data = array(
			'tasty_links_tabs' => array(
				array(
					'title'  => __( 'Settings', 'tasty-links' ),
					'url'    => add_query_arg( array( 'tab' => 'settings' ), menu_page_url( 'tasty-links', false ) ),
					'active' => self::is_current_page( 'settings' ),
				),
				array(
					'title'  => __( 'Links', 'tasty-links' ),
					'url'    => admin_url( 'edit.php?post_type=tasty_link' ),
					'active' => self::is_current_page( 'tasty_link' ),
				),
				array(
					'title'  => __( 'About', 'tasty-links' ),
					'url'    => add_query_arg( array( 'tab' => 'about' ), menu_page_url( 'tasty-links', false ) ),
					'active' => self::is_current_page( 'about' ),
				),
			),
		);
		self::show_template_part( 'admin-header', $data );

		do_action( 'tasty_links_admin_header' );
	}

	/**
	 * Add admin footer template.
	 *
	 * @return void
	 */
	public static function add_admin_footer() {
		if ( ! self::is_links_admin() ) {
			return;
		}
		self::show_template_part( 'admin-footer' );
	}

	/**
	 * Make links custom post type pages act as ours.
	 *
	 * @param bool $is_ours Current page is ours or not.
	 *
	 * @return bool
	 */
	public static function make_links_cpt_menu_item_ours( $is_ours ) {
		if ( self::is_current_page( 'tasty_link' ) ) {
			return true;
		}
		return $is_ours;
	}

	/**
	 * Show license notice in tasty links custom post type page.
	 *
	 * @param bool   $show_notice Show notice status.
	 * @param object $plugin Current plugin.
	 *
	 * @return bool|mixed
	 */
	public static function show_notice_in_links_cpt( $show_notice, $plugin ) {
		if ( $show_notice || 'tasty-links' !== $plugin->get_plugin_slug() ) {
			return $show_notice;
		}
		return 'edit-tasty_link' === get_current_screen()->id;
	}

	/**
	 * Enqueue assets.
	 *
	 * @return void
	 */
	public static function enqueue_assets() {
		if ( ! self::is_links_admin() ) {
			return;
		}

		wp_enqueue_style( 'tasty-links-common', plugins_url( 'assets', __DIR__ ) . '/css/common.css', array(), TASTY_LINKS_PLUGIN_VERSION );
	}

	/**
	 * Workaround to make CPT pages act as tasty menu active items.
	 *
	 * @return void
	 */
	public static function handle_cpt_menu_item_classes() {
		global $menu, $submenu;

		if ( ! isset( $submenu['tasty'] ) ) {
			return;
		}

		foreach ( $menu as $cur_menu_key => $cur_menu ) {
			if ( 'tasty' !== $cur_menu[2] ) {
				continue;
			}
			if ( 'tasty_link' !== get_current_screen()->post_type ) {
				continue;
			}
			$menu[ $cur_menu_key ][4] = 'wp-has-current-submenu';// phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		}

		foreach ( $submenu['tasty'] as $sub_key => $sub_item ) {
			if ( 'tasty-links' !== $sub_item[2] ) {
				continue;
			}

			if ( 'tasty_link' !== get_current_screen()->post_type ) {
				continue;
			}

			$submenu['tasty'][ $sub_key ][4] = 'current';// phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
		}
	}

	/**
	 * Gets all post types that links can be replaced.
	 *
	 * @return array
	 */
	private static function get_link_locations() {
		$post_types = array();
		self::add_public_post_types( $post_types );
		self::add_custom_post_types( $post_types );
		self::order_post_types( $post_types );
		return $post_types;
	}

	/**
	 * Add public post types to the list of post types that can be autolinked.
	 *
	 * @since 1.3
	 *
	 * @param array $post_types Post types to add to.
	 * @return void
	 */
	private static function add_public_post_types( &$post_types ) {
		$public_post_types = get_post_types(
			array(
				'public' => true,
			),
			'objects'
		);

		foreach ( $public_post_types as $post_type => $object ) {
			if ( post_type_supports( $post_type, 'editor' ) ) {
				$post_types[ $post_type ] = $object->label;
			}
		}
	}

	/**
	 * Add custom post types to the list of post types that can be autolinked.
	 *
	 * @since 1.3
	 *
	 * @param array $post_types Post types to add to.
	 * @return void
	 */
	private static function add_custom_post_types( &$post_types ) {
		$allowed = self::allowed_nonpublic_post_types();

		foreach ( $allowed as $post_type => $label ) {
			if ( post_type_exists( $post_type ) ) {
				$post_types[ $post_type ] = $label;
			}
		}
	}

	/**
	 * Gets all non public post types that can be autolinked.
	 *
	 * @since 1.3
	 *
	 * @return array
	 */
	private static function allowed_nonpublic_post_types() {
		$allowed = array(
			'tasty_recipe' => 'Tasty Recipes',
			'wprm_recipe'  => 'WP Recipe Maker', // These both use 'Recipe' for the name.
		);

		/**
		 * Allow non-public post types to be used for auto-linking.
		 *
		 * @since 1.3
		 *
		 * @param array $allowed Array of post types.
		 * @return array
		 */
		return apply_filters( 'tasty_links_nonpublic_post_types', $allowed );
	}

	/**
	 * Alphabetically order the post types and add All as the first option.
	 *
	 * @since 1.3
	 *
	 * @param array $post_types Post types to order.
	 * @return void
	 */
	private static function order_post_types( &$post_types ) {
		asort( $post_types ); // Order by name.

		// Add 'All' as the first option.
		$all        = array( 'all' => __( 'All', 'tasty-links' ) );
		$post_types = array_merge( $all, $post_types );
	}
}
