<?php
/**
 * Enhancements to the WordPress REST API.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links;

/**
 * Enhancements to the WordPress REST API.
 */
class REST_API {

	/**
	 * Hooks up the 'rest_prepare_{$post_type}' filter for each wp/v2 post type.
	 */
	public static function action_rest_api_init() {
		foreach ( get_post_types( array( 'show_in_rest' => true ), 'names' ) as $post_type ) {
			add_filter( "rest_prepare_{$post_type}", array( __CLASS__, 'filter_rest_prepare_post' ), 10, 2 );
		}
	}

	/**
	 * Filter raw content prior to returning in the REST API.
	 *
	 * @param object $response Prepared response.
	 * @param object $post     Post to be returned.
	 * @return object
	 */
	public static function filter_rest_prepare_post( $response, $post ) {
		$data = $response->get_data();
		if ( empty( $data['content']['raw'] ) ) {
			return $response;
		}

		if ( ! tasty_links_should_be_applied( 'rest-api' ) || Frontend::should_disable_auto_linking( $post->post_type ) ) {
			return $response;
		}

		$skipped_links          = array(
			get_permalink( $post->ID ),
		);
		$data['content']['raw'] = Frontend::apply_links_to_content( $data['content']['raw'], 'editor', null, $skipped_links );
		// Gutenberg doesn't support HTML comments.
		// See https://github.com/WordPress/gutenberg/issues/13318.
		$content = $data['content']['raw'];
		if ( preg_match_all( Frontend::IGNORE_CONTENT_REGEX, $content, $matches, PREG_OFFSET_CAPTURE ) ) {
			$diff = 0;
			foreach ( $matches[0] as $match ) {
				$match[1] = $match[1] + $diff;
				$before   = substr( $content, 0, $match[1] );
				// Ignore content inside a html block.
				// <!-- tasty-links-ignore -->These apples<!-- /tasty-links-ignore -->.
				preg_match_all( '#<!--\s?\/?wp:html\s?-->#', $before, $before_matches );
				if ( ! empty( $before_matches[0] ) ) {
					$last_before = array_pop( $before_matches[0] );
					// If the last marker wasn't a closing marker, then we're
					// inside the html block.
					if ( false === stripos( $last_before, '/wp:html' ) ) {
						continue;
					}
				}
				if ( false !== stripos( $match[0], '/tasty-links-ignore' ) ) {
					$replacement = '</span>';
				} else {
					$replacement = '<span class="tasty-link" data-tasty-link-remove="1">';
				}

				$content = substr_replace( $content, $replacement, $match[1], strlen( $match[0] ) );
				$diff    = $diff + strlen( $replacement ) - strlen( $match[0] );
			}
		}
		$data['content']['raw'] = $content;
		$response->set_data( $data );
		return $response;
	}
}
