<?php
/**
 * Integration base for a plugin with auto linking.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Integrations;

use Tasty_Links\Admin;
use Tasty_Links\Frontend;

/**
 * Integration base for a plugin with auto linking.
 */
class Base_Autolink {

	/**
	 * The setting name for this integration.
	 *
	 * @since 1.3
	 * @var string
	 */
	protected static $setting_name = '';

	/**
	 * In the admin area, show all possible links so they can be removed.
	 *
	 * @since 1.3
	 *
	 * @param bool $is_admin Whether or not we're in the admin.
	 * @return array
	 */
	protected static function set_link_reset( $is_admin ) {
		$pass_atts = array();
		if ( $is_admin ) {
			$pass_atts['reset_limit'] = apply_filters( 'tasty_links_reset_limit', true );

			// The filter is needed to remove the limit for Tasty Recipes modal editor.
			add_filter( 'tasty_links_reset_limit', '__return_false' );
		}
		return $pass_atts;
	}

	/**
	 * Don't add the link if it's to the current page.
	 *
	 * @since 1.3
	 *
	 * @return array
	 */
	protected static function skipped_links() {
		global $post, $wp;

		$skipped_links = array();
		if ( $post ) {
			$skipped_links[] = get_permalink( $post->ID );
		}
		if ( ! empty( $wp->request ) ) {
			$skipped_links[] = home_url( $wp->request );
		}
		return $skipped_links;
	}

	/**
	 * Whether or not this integration is disabled.
	 *
	 * @since 1.3
	 * @return bool
	 */
	public static function is_disabled() {
		return Frontend::should_disable_auto_linking( static::$setting_name );
	}
}
