<?php
/**
 * Integration points with WPRM.
 *
 * @package Tasty_Links
 */

namespace Tasty_Links\Integrations;

use Tasty_Links\Frontend;

/**
 * Integration with WPRM to allow manual link removal,
 * and show links in the admin editor.
 */
class Wprm extends Base_Autolink {

	/**
	 * The setting name for this integration.
	 *
	 * @since 1.3
	 * @var string
	 */
	protected static $setting_name = 'wprm_recipe';

	/**
	 * Show links in the admin editor and front-end.
	 *
	 * @since 1.3
	 * @param string $output Editor content.
	 *
	 * @return string
	 */
	public static function filter_wprm_content( $output ) {
		if ( self::is_disabled() ) {
			return $output;
		}

		$context = wp_is_json_request() ? 'editor' : 'frontend';
		$pass_me = self::set_link_reset( $context === 'editor' );
		$output  = Frontend::apply_links_to_content( $output, $context, false, $pass_me );
		return $output;
	}

	/**
	 * Show the link in the editor modal so it can be removed.
	 * This only works in rich text fields.
	 *
	 * @since 1.3
	 * @param array $data Recipe data.
	 *
	 * @return array
	 */
	public static function filter_wprm_recipe_data( $data ) {
		if ( self::is_disabled() ) {
			return $data;
		}

		$rich_text = array(
			'notes',
		);
		foreach ( $rich_text as $setting ) {
			if ( empty( $data[ $setting ] ) || ! is_string( $data[ $setting ] ) ) {
				continue;
			}

			$pass_me          = self::set_link_reset( true );
			$data[ $setting ] = Frontend::apply_links_to_content( $data[ $setting ], 'editor', $pass_me );
		}
		return $data;
	}
}
