<?php
/**
 * Displays equipment in a recipe card.
 *
 * @package Tasty_Links
 */

use Tasty_Links\Admin;

if ( empty( $equipment_ids ) ) {
	return;
}

global $post;
$backup_post = $post;

$image_size = apply_filters( 'tasty_links_card_image_size', 'thumbnail' );

foreach ( $equipment_ids as $equipment_id ) :
	$this_post = get_post( $equipment_id );
	if ( ! $this_post ) {
		continue;
	}

	// phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
	$post = $this_post;

	$link_start    = '';
	$tlink         = get_post_meta( $equipment_id, Admin::LINK_KEY, true );
	$rel_nofollow  = get_post_meta( $equipment_id, Admin::REL_NOFOLLOW_META_KEY, true );
	$rel_sponsored = get_post_meta( $equipment_id, Admin::REL_SPONSORED_META_KEY, true );
	$open_new      = get_post_meta( $equipment_id, Admin::OPEN_NEW_META_KEY, true );
	if ( $tlink ) {
		$link_start = '<a class="tasty-link" ';
		$rel_value  = '';
		// rel="nofollow" is on by default.
		if ( 'off' !== $rel_nofollow ) {
			$rel_value .= ' nofollow';
		}
		// rel="sponsored" is off by default.
		if ( 'on' === $rel_sponsored ) {
			$rel_value .= ' sponsored';
		}
		// Add 'noopener' to external domains.
		if ( 'off' !== $open_new && wp_parse_url( home_url(), PHP_URL_HOST ) !== wp_parse_url( $tlink ) ) {
			$rel_value .= ' noopener';
		}
		if ( ! empty( $rel_value ) ) {
			$link_start .= 'rel="' . esc_attr( trim( $rel_value ) ) . '" ';
		}
		if ( 'off' !== $open_new ) {
			$link_start .= 'target="_blank" ';
		}
		$link_start .= 'href="' . esc_url( $tlink ) . '">';
	}
	?>
	<article class="tasty-link-card">
		<?php
		if ( has_post_thumbnail( $this_post ) ) {
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $link_start;
			echo get_the_post_thumbnail(
				$this_post,
				$image_size,
				array(
					'data-pin-nopin' => 'true',
				)
			);

			echo $link_start ? '</a>' : '';
		}

		echo '<p>';
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $link_start;
		echo esc_html( apply_filters( 'the_title', $this_post->post_title, $this_post->ID ) );
		echo ( $link_start ? '</a>' : '' );
		echo '</p>' . PHP_EOL;

		if ( $link_start ) :
				echo '<span>';
				// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo $link_start . esc_html__( 'Buy Now →', 'tasty-links' ) . '</a>';
				echo '</span>';
			?>
		<?php endif; ?>
	</article>
	<?php
endforeach;

// phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
$post = $backup_post;
